"""The 'worktoy.loremify' package provides a Loremify class for generating
lorem ipsum text!"""
#  AGPL-3.0 license
#  Copyright (c) 2024 Asger Jon Vistisen
from __future__ import annotations

from ._vocabulary import Vocabulary
